const cors = require('cors')
const hsts = require('hsts')
const compression = require('compression')
const bodyParser = require('body-parser')
const bearerToken = require('express-bearer-token')
const cookieParser = require('cookie-parser')
const contextService = require('request-context')
const { BAD_REQUEST, INTERNAL_SERVER_ERROR, NOT_FOUND, UNAUTHORIZED, FORBIDDEN } = require('http-status-codes')

const router = require('../routes')
const API_PREFIX = '/'
 
const { ClientError, AuthorizationError, NotFoundError, FinanceApiError, ForbiddenError } = require('../../errors')
const { JsonWebTokenError } = require('jsonwebtoken')
//const { winstonLogger, winstonErrorLogger } = require('./loggingMiddleware')
const { expressValidator, addValidateToRequest } = require('./validationMiddleware')
//const { resolveLoggingData } = require('./resolveLoggingDataMiddleware')
const { ValidationError } = require('../../errors')
const { getResourceString } = require('../lang')
const getResourceStringDE = getResourceString('de-de')

const JSONBodyParserMiddleware = bodyParser.raw({ type: '*/*' })
const RawBodyParserMiddleware = bodyParser.json({ type: 'application/json', limit: '2mb' })


const initializeApiMiddleware = app => {
  app.use(API_PREFIX, [
    bearerToken({ reqKey: 'token' }),
    contextService.middleware('request'),
    cors(),
   // JSONBodyParserMiddleware(),
  
    // parse cookies
    cookieParser(),
 function(req,res,next){
    RawBodyParserMiddleware(req,res,next)
 },
    router(),

    // initialize express-validator and add `validate` method to requests
    //expressValidator,
    addValidateToRequest,
  ]) 
}

module.exports = initializeApiMiddleware
