const validator = require('express-validator')
const { isNumber, isString, isArray, isPlainObject, isNull } = require('lodash')
const { isEmailList, isValidFileName, isUrl } = require('../_helpers/additionalCustomExpressValidators')
const { ValidationError } = require('../../errors')
const { getResourceString } = require('../lang')
const getResourceStringDE = getResourceString('de-de')

const toObject = (obj, err) => {
  obj[err.param] = obj[err.param] || []
  if (typeof err.msg === 'object') {
    obj[err.param].push(err.msg)
  } else {
    obj[err.param].push({ code: err.msg, message: getResourceStringDE('ERROR_INPUT_VALIDATION_FAILED') })
  }
  return obj
}
 
/*const expressValidator = validator({
  customValidators: {
    isNumber,
    isString,
    isArray, 
    isEmailList,
    isValidFileName,
    isUrl,
    isObject: isPlainObject,
    greaterThan: (param, number) => param > number,
    isNullOrInt: (value, options) => {
      return isNull(value) || validator.validator.isInt(value, options)
    }
  },
  customSanitizers: {
    getFirstString: function (value) {
      if (!value) {
        return value
      }
      const sanitized = value.split(' ', 1)
      return sanitized[0]
    }
  }
})*/

const addValidateToRequest = (req, res, next) => {
  req.validate = async schema => {
    try {
      req.check(schema)
      const result = await req.getValidationResult()
      return result.isEmpty() ? false : new ValidationError(result.array().reduce(toObject, {}))
    } catch (err) {
      return err
    }
  }

  next()
}

module.exports = {
  addValidateToRequest,
  //expressValidator
}
