const faker = require('faker/locale/de')
const moment = require('moment')
const { randomBytes } = require('../../_helpers/additionalCustomExpressValidators')
//const { getDefaultInvoiceColumns, getDefaultInvoiceInfoSectionFields, getDefaultInvoiceInfoSectionCustomFields } = require('../../models/_defaults')
const { INVOICE_STATE_DRAFT, CUSTOMER_KIND_COMPANY, PRICEKIND_NET, SMALLBUSINESS_TEXT, POSITION_META_CUSTOM } = require('../../../constants')

const defaultTexts = {
  introduction: '',
  conclusion: ''
}

const invoiceCustomerData = customer => {
  if (customer.kind === CUSTOMER_KIND_COMPANY) {
    return {
      kind: CUSTOMER_KIND_COMPANY,
      companyName: customer.companyName + ' ' + customer.companyNameAffix,
      name: customer.name,
      street: customer.address.street,
      zipCode: customer.address.zipCode,
      city: customer.address.city,
      country: customer.address.country,
      countryIso: customer.address.countryCode,
      number: customer.number
    }
  } else { 
    return {
      kind: customer.kind,
      firstName: customer.firstName,
      lastName: customer.lastName,
      name: '',
      street: customer.address.street,
      zipCode: customer.address.zipCode,
      city: customer.address.city,
      country: customer.address.country,
      countryIso: customer.countryCode,
      salutation: customer.salutation,
      title: '', // to be checked
      number: customer.number
    }
  }
}

const buildInvoicePosition = tenantId => {
  const priceNet = faker.random.number({ min: 10, max: 600 })
  const vatPercent = faker.random.number({ min: 1, max: 2 }) === 1 ? 19 : 7
  const mock = {
    // offerId: data.offerId,
    tenantId,
    title: faker.commerce.productName(),
    description: faker.commerce.productAdjective(),
    showDescription: faker.random.boolean(),
    amount: faker.random.number({ min: 1, max: 50 }),
    unit: 'Stk.',
    discountPercent: 10,
    vat: 19,
    vatPercent,
    priceGross: priceNet * (+vatPercent / 100),
    priceNet,
    sortIndex: 0,
    metaData: { type: POSITION_META_CUSTOM }
  }
  return mock
}
const positions = tenantId => {
  const positions = []
  for (let i = 0; i < 4; i++) {
    positions.push(buildInvoicePosition(tenantId))
  }
  return positions
}

const buildInvoice = (tenantId, customerData) => {
  const mock = {
    tenantId,
    payConditionData: {
      name: 'Sofort',
      dueDays: 0,
      invoiceText: 'Zahlbar nach Erhalt der Rechnung',
      offerText: 'Zahlbar nach Erhalt der Rechnung',
      discountDays: 0,
      discountPercentage: 0,
      isBasic: true,
      isInstant: true
    },

    state: INVOICE_STATE_DRAFT,
    cashDiscountSetting: null,
    priceKind: PRICEKIND_NET,
    date: moment().toISOString(),
    deliveryDate: moment().toISOString(),
    dueToDate: moment()
      .add(14, 'day')
      .toISOString(),
    smallBusiness: false,
    smallBusinessText: SMALLBUSINESS_TEXT,
    title: '',
    texts: Object.assign({}, defaultTexts),
    notes: faker.lorem.words(faker.random.number({ min: 0, max: 20 })),
    customerData: invoiceCustomerData(customerData),
    columns: getDefaultInvoiceColumns('DE'),
    infoSectionFields: getDefaultInvoiceInfoSectionFields('DE'),
    infoSectionCustomFields: getDefaultInvoiceInfoSectionCustomFields('DE'),
    lockedAt: null,
    number: randomBytes(10),
    // calculated values
    totalNet: faker.random.number({ min: 10, max: 600 }),
    totalGross: faker.random.number({ min: 10, max: 600 }),
    vat: '19',
    outstandingAmount: '100',
    cancellationDate: null,
    recurringInvoiceId: null,
    type: 'invoice', // to be set
    invoizPayData: {},
    autoDunningEnabled: false,
    overDueNotificationSent: false,
    printCustomDocument: false,
    customInvoiceDocumentId: null,
    positions: positions(tenantId)
  }

  if (mock.smallBusiness) {
    const vatColumn = mock.columns.find(({ name }) => {
      return name === 'vat'
    })
    if (vatColumn.active) vatColumn.active = false
  }

  return mock
}

const buildMockInvoice = (tenantId, customerData) => {
  const invoiceData = buildInvoice(tenantId, customerData)
  return invoiceData
}

module.exports = buildMockInvoice
