const faker = require('faker/locale/de')
const moment = require('moment')
const { randomBytes } = require('../../_helpers/randomBytes')
//const { getDefaultOfferColumns, getDefaultOfferInfoSectionFields, getDefaultOfferInfoSectionCustomFields } = require('../../models/_defaults')
const {
  CUSTOMER_KIND_COMPANY,
  OFFER_STATE_OPEN,
  PRICEKIND_NET,
  SMALLBUSINESS_TEXT,
  POSITION_META_CUSTOM,
  OFFER_TYPE_STANDARD
} = require('../../../constants')

const defaultTexts = {
  introduction: '',
  conclusion: ''
}

const customerData = customer => {
  if (customer.kind === CUSTOMER_KIND_COMPANY) {
    return {
      kind: CUSTOMER_KIND_COMPANY,
      companyName: customer.companyName + ' ' + customer.companyNameAffix,
      name: customer.name,
      street: customer.address.street,
      zipCode: customer.address.zipCode,
      city: customer.address.city,
      country: customer.address.country,
      countryIso: customer.address.countryCode,
      number: customer.number
    }
  } else {
    return {
      kind: customer.kind,
      firstName: customer.firstName,
      lastName: customer.lastName,
      name: '',
      street: customer.address.street,
      zipCode: customer.address.zipCode,
      city: customer.address.city,
      country: customer.address.country,
      countryIso: customer.countryCode,
      salutation: customer.salutation,
      title: '', // to be checked
      number: customer.number
    }
  }
}
const buildOfferPosition = tenantId => {
  const priceNet = faker.random.number({ min: 10, max: 600 })
  const vatPercent = faker.random.number({ min: 1, max: 2 }) === 1 ? 19 : 7
  const mock = {
    tenantId,
    title: faker.commerce.productName(),
    description: faker.commerce.productAdjective(),
    showDescription: faker.random.boolean(),
    amount: faker.random.number({ min: 1, max: 50 }),
    unit: 'Stk.',
    discountPercent: 10,
    vat: 19,
    vatPercent,
    priceGross: priceNet * (+vatPercent / 100),
    priceNet,
    sortIndex: 0,
    metaData: { type: POSITION_META_CUSTOM }
  }
  return mock
}
const positions = tenantId => {
  const positions = []
  for (let i = 0; i < 4; i++) {
    positions.push(buildOfferPosition(tenantId))
  }
  return positions
}

const buildOffer = (tenantId, customer) => {
  return {
    tenantId,
    type: OFFER_TYPE_STANDARD,
    state: OFFER_STATE_OPEN,
    priceKind: PRICEKIND_NET,
    date: moment().toISOString(),
    smallBusiness: false,
    smallBusinessText: SMALLBUSINESS_TEXT,
    texts: Object.assign({}, defaultTexts),
    title: 'Angebot',
    notes: faker.lorem.words(faker.random.number({ min: 0, max: 20 })),
    customerData: customerData(customer),
    columns: getDefaultOfferColumns('DE'),
    infoSectionFields: getDefaultOfferInfoSectionFields('DE'),
    infoSectionCustomFields: getDefaultOfferInfoSectionCustomFields('DE'),
    isLocked: false,
    deliveryConditionData: {
      name: 'Frei Haus',
      text: 'Die Lieferung erfolgt frei Haus.'
    },
    payConditionData: {
      name: 'Sofort',
      dueDays: 0,
      invoiceText: 'Zahlbar nach Erhalt der Rechnung',
      offerText: 'Zahlbar nach Erhalt der Rechnung',
      discountDays: 0,
      discountPercentage: 0,
      isBasic: true,
      isInstant: true
    },
    number: randomBytes(10),
    // calculated values
    totalNet: faker.random.number({ min: 10, max: 600 }),
    totalGross: faker.random.number({ min: 10, max: 600 }), // add vat
    vat: '19',
    printCustomDocument: false,
    customOfferDocumentId: null,
    positions: positions(tenantId)
  }
}

const buildMockOffer = (tenantId, customer) => {
  const offer = buildOffer(tenantId, customer)

  return offer
}

module.exports = {
  buildMockOffer,
  buildOfferPosition
}
